function [outsid,NCcom]=NC_sid_sweep_base(bsd_corr,direc,freq,NCcom)
% basic sid_sweep 
%
%   [outsid,NCcom]=NC_sid_sweep_base(bsd_corr,direc,freq,NCcom)
%
%   bsd_corr corrected bsd (by e.g. NC_bsdcorr)
%   direc    direction structure ('nodirec' direction absent)
%   freq     candidate frequency;
%             if 2-dim freq(2), spin-down
%             if 3-dim freq(3) is the semiband; def semiband 0.5 Hz
%   NCcom
%      sband    search band (in units of 1/SD; default 20) sband=0 -> 20
%      interl   interlacing delay (in 1/SD); default sband/2, if sband = 20, interl = 5
%      enl      enlargement factor (def 1)
%      nsid     number of sidereal bins (def or 0 = round(SD/outsid.DT))
%
%      sol      if exists and >0 -> also solar (NYI)
%      checksolar  if exist and = 1, substitutes SD with SolD
%                  if > 1, substitutes SD with checksolar
%
%      nweek    if present and >0, t-fr mode, n week for pieces (typically 1)
%      dweek    week interlacing delay (<= nweek, def nweek/2, =nweek or 0 no interlacing)
%
%      post     postfilter
%               == 0 no
%               == 1 basic (rectangular length round(sband/5)
%               otherwise the MA filter
%
%      out.ff   1 all, or indices [ini fin]
%      out.POW  1 all, or indices [ini fin]

% Snag Version 2.0 - Apr 2020
% Part of Snag toolbox - Signal and Noise for Gravitational Antennas
% by O.J.Piccinni, S.D'Antonio and S.Frasca - ser.fras@gmail.com
% Department of Physics - Sapienza University - Rome

%% base parameters

ticini=tic;

SD=86164.09053083288;
SolD=86400;

if ~exist('NCcom','var')
    NCcom=struct();
end

if isfield(NCcom,'checksolar')
    if NCcom.checksolar == 1
        SD=SolD;
    elseif NCcom.checksolar > 1
        SD=checksolar;
    end
    outsid.CHECKSOLAR=checksolar;
end

outsid.start=datetime('now');

if isfield(NCcom,'out')
    if isfield(NCcom.out,'ff')
        out.ff=NCcom.out.ff;
    else
        out.ff=0;
    end
    if isfield(NCcom.out,'POW')
        out.POW=NCcom.out.POW;
    else
        out.POW=0;
    end
else
    out.ff=0;
    out.POW=0;
end

% bsd_corr parameters

y=y_gd(bsd_corr);

dt=dx_gd(bsd_corr);
n=n_gd(bsd_corr);
Tobs=n*dt/86400;
dfr=1/(n*dt);
cont=cont_gd(bsd_corr);
t0=cont.t0;
inifr=cont.inifr;
bandw=cont.bandw;
ant=bsd_ant(bsd_corr);

% freq and band

if length(freq) == 3
    semiband=freq(3);
else
    semiband=0.5;
end

band=[freq(1)-semiband freq(1)+semiband];
band(1)=max(band(1),inifr);
band(2)=min(band(2),inifr+bandw);
iband(1)=round((band(1)-inifr)/dfr)+1;
iband(2)=round((band(2)-inifr)/dfr);
i1=iband(1);
i2=iband(2);
band=(iband-1)*dfr+inifr;

% NCcom - I

NCcom.MODE='';

if isfield(NCcom,'sband')
    sband=NCcom.sband;
    if sband == 0
        sband=20;
    end
else
    sband=20;
end

if isfield(NCcom,'interl')
    interl=NCcom.interl;
else
    if sband == 20
        interl=5;
    else
        interl=sband/2;
    end 
end

if isfield(NCcom,'enl')
    enl=NCcom.enl;
else
    enl=1;
end

if isfield(NCcom,'sol')
    if NCcom.sol > 0
        icsol=1;
    end
else
    icsol=0;
end


% DFR,k2

DFR=round(interl/SD/dfr)*dfr;
k2=round(DFR/dfr);

% harmonics weight

if isstruct(direc)
    sidpat_rand=ana_sidpat_rand(ant,direc,0,4);
    weig=mean(abs(sidpat_rand.s(2:5,:)').^2);
    weig=weig/mean(weig);
else
    weig=[1 1 1 1]';
    direc.a=0;
end

% microband,k1

microband0=sband/SD;
microband=round(microband0/(2*dfr))*2*dfr;

k1=round(microband/dfr);
K1=round(k1*enl);

N=floor((i2-i1+1-k1)/k2)+1;

fr=band(1)+(0:N-1)*DFR+microband/2;

DT=1/(microband*enl);
maxsidbins=SD/DT;


% NCcom - II

if isfield(NCcom,'nsid')
    nsid=NCcom.nsid;
else
    nsid=round(maxsidbins);
end

if isfield(NCcom,'nweek')
    nweek=NCcom.nweek;
else
    nweek=0;
end

if isfield(NCcom,'dweek')
    dweek=NCcom.dweek;
    if dweek ==0
        dweek=nweek;
    end
else
    dweek=nweek/2;
end

NW=round(7*nweek*nsid);
dNW=round(7*dweek*nsid);

if nweek == 0
    NCcom.MODE=[NCcom.MODE '  ' 'frequency mode'];
else
    NCcom.MODE=[NCcom.MODE '  ' 'time-frequency mode'];
    Nweek=floor(Tobs/(dweek*7))-1;
end

if isfield(NCcom,'post')
    post=NCcom.post;
else
    post=1;
end
lfilt=length(post);
if lfilt == 1
    if post == 1;
        lfilt=round(sband/5);
        post=ones(1,lfilt)/lfilt;
    end
    if post == 0;
        post=1;
    end
end

totpost=sum(post);
frdel=DFR*sum(post.*(0:lfilt-1))/totpost;

wnois=min(floor(nsid/2),21);

% outsid

outsid.ant=ant;
outsid.t0=t0;
outsid.dt=dt;
outsid.n=n;
outsid.Tobs=Tobs;
outsid.inifr=inifr;
outsid.bandw=bandw;
outsid.dfr=dfr;
outsid.band=band;
outsid.searband=sband(1);
outsid.interl=interl;
outsid.enl=enl;
outsid.i1=i1;
outsid.i2=i2;
outsid.weig=weig;
outsid.wnois=wnois;
outsid.microband=microband;
outsid.DFR=DFR;
outsid.DT=DT;
outsid.maxsidbins=maxsidbins;
outsid.nsid=nsid;
outsid.nweek=nweek;
outsid.dweek=dweek;
if nweek > 0
    outsid.Nweek=Nweek;
end
outsid.NW=NW;
outsid.dNW=dNW;
outsid.post=post;
outsid.frdel=frdel;
outsid.fr=fr-frdel;
outsid.k1=k1;
outsid.K1=K1;
outsid.k2=k2;
outsid.N=N; %NCcom,outsid

toc(ticini)

%% base matrix

% k1 len in bin
% k2 step in bin
% N  number of steps

f=fft(y);
f=f(i1:i2);

jj1=round((0:N-1)*k2)+1;
jj2=jj1+k1-1;
Y=zeros(k1,N);

for i = 1:N
    Y(:,i)=f(jj1(i):jj2(i));
end
Y=ifft(Y,K1);

outsid.Y1=mean(abs(Y').^2); 
outsid.Y2=mean(abs(Y).^2); 

toc(ticini)

%% sid hours

t=(0:K1-1)'*dt*n/K1;

st0=loc_sidtim(t0,ant.long)-direc.a/15;
ww=zeros(1,nsid);

zer=cont.tfstr.zeros/86400;
vin=ones(1,K1);
cover=1-zero_pers(vin,0,t(2)-t(1),zer)';
outsid.cover=cover;

Y=Y.*cover;
Y=abs(Y).^2; size(Y)

st=mod(t+st0*3600,SD);
st=floor(st*nsid/SD)+1; % all sid bins

switch nweek
    case 0
%% noweek
        for ij = 1:nsid
             ii=find(st == ij);
             ww(ij)=mean(cover(ii));
        end

        jjj=isnan(ww);
        ww(jjj)=1;
        outsid.ww=ww;

        PP=zeros(nsid,N);
        for ij = 1:nsid
             ii=st == ij;
             PP(ij,:)=mean(Y(ii,:));
        end

        jjj=isnan(PP);
        PP(jjj)=0;
        POW=PP./ww';

        for jk=1:N
             jn=find(POW(:,jk)>0);
             j0=find(POW(:,jk)==0);
             if ~isempty(j0)
                 if(~isempty(jn))
                     POW(j0,jk)=mean(POW(jn,jk));
                 end
             end
        end

        ff=fft(POW);

        if out.POW(1) > 0
            if lenght(out.POW > 1)
                outsid.POW=POW(:,out.POW(1):out.POW(2));
            else
                outsid.POW=POW;
            end
        end
        clear POW
        if out.ff(1) > 0
            if lenght(out.ff > 1)
                outsid.ff=ff(:,out.ff(1):out.ff(2));
            else
                outsid.ff=ff;
            end
        end

        sidsig=sum((abs(ff(2:5,:)).^2).*weig');
        sidnois=mean(abs(ff(6:wnois,:)).^2);
        sidzer=abs(ff(1,:)).^2;
        sidsig=filter(post,1,sidsig);
        sidnois=filter(post,1,sidnois);
        sidzer=filter(post,1,sidzer);
        outsid.sidsig=sidsig;
        outsid.sidnois=sidnois;
        outsid.sidzer=sidzer;
    otherwise
%% week
        sidsigs=zeros(Nweek,N);
        sidnoiss=sidsigs;
        jw1=1;
        jw2=NW;
        
        for jw = 1:Nweek
            cover1=cover(jw1:jw2);
            st1=st(jw1:jw2);
            Y1=Y(jw1:jw2,:); 
            for ij = 1:nsid
                 ii=find(st1 == ij);
                 ww(ij)=mean(cover1(ii));
            end

            jjj=isnan(ww);
            ww(jjj)=1;
            outsid.ww(jw,:)=ww;

            PP=zeros(nsid,N);
            for ij = 1:nsid
                 ii=st1== ij;
                 PP(ij,:)=mean(Y1(ii,:));
            end

            jjj=isnan(PP);
            PP(jjj)=0;
            POW=PP./ww';

            for jk=1:N
                 jn=find(POW(:,jk)>0);
                 j0=find(POW(:,jk)==0);
                 if ~isempty(j0)
                     if(~isempty(jn))
                         POW(j0,jk)=mean(POW(jn,jk));
                     end
                 end
            end

            ff=fft(POW);

            if out.POW(1) > 0
                if lenght(out.POW > 1)
                    outsid.POW=POW(:,out.POW(1):out.POW(2));
                else
                    outsid.POW=POW;
                end
            end
            clear POW
            if out.ff(1) > 0
                if lenght(out.ff > 1)
                    outsid.ff=ff(:,out.ff(1):out.ff(2));
                else
                    outsid.ff=ff;
                end
            end
            
            sidsigs(jw,:)=sum((abs(ff(2:5,:)).^2).*weig');
            sidnoiss(jw,:)=mean(abs(ff(6:wnois,:)).^2);
            sidzers(jw,:)=abs(ff(1,:)).^2;
            
            jw1=jw1+dNW;
            jw2=jw2+dNW;
        end
        outsid.sidsigs=sidsigs;
        outsid.sidnoiss=sidnoiss;
        outsid.sidzers=sidzers;
end

%% out data

tocout=toc(ticini);
outsid.tictoc=tocout;
